//====================================================================================//
//                                                                                    //
//                                 Class Booleans                                     //
//                                                                                    //
//====================================================================================//
//  This File:   bools.C                    Language: C++  (xlC and CC)               //
//  Software:    PEPS 3.1                    Advisor: Brigitte.Plateau@imag.fr        //
//  Error Codes: 2100                                                                 //
//====================================================================================//
//  Creation:    19/dec/95                        by: Paulo.Fernandes@imag.fr         //
//  Last Change: 14/feb/00                        by: Paulo.Fernandes@imag.fr         //
//====================================================================================//
#include "peps.h"

//====================================================================================//
//---------------------------- Machine Dependent Constants ---------------------------//
//====================================================================================//

     //--------------------------------------------------------------------------//
     //      Static variable with  0000000001 , 0000000010 , 0000000100 ...      //
     //--------------------------------------------------------------------------//
unsigned int bools::mask0010[word_size] = 
   {            1U ,             2U ,             4U ,             8U ,
               16U ,            32U ,            64U ,           128U ,
              256U ,           512U ,          1024U ,          2048U ,
             4096U ,          8192U ,         16384U ,         32768U ,
            65536U ,        131072U ,        262144U ,        524288U ,
          1048576U ,       2097152U ,       4194304U ,       8388608U ,
         16777216U ,      33554432U ,      67108864U ,     134217728U ,
        268435456U ,     536870912U ,    1073741824U ,    2147483648U         };
     //--------------------------------------------------------------------------//
     //      Static variable with  1111111110 , 1111111101 , 1111111011 ...      //
     //--------------------------------------------------------------------------//
unsigned int bools::mask1101[word_size] = 
   {   4294967294U ,    4294967293U ,    4294967291U ,    4294967287U ,
       4294967279U ,    4294967263U ,    4294967231U ,    4294967167U ,
       4294967039U ,    4294966783U ,    4294966271U ,    4294965247U ,
       4294963199U ,    4294959103U ,    4294950911U ,    4294934527U ,
       4294901759U ,    4294836223U ,    4294705151U ,    4294443007U ,
       4293918719U ,    4292870143U ,    4290772991U ,    4286578687U ,
       4278190079U ,    4261412863U ,    4227858431U ,    4160749567U ,
       4026531839U ,    3758096383U ,    3221225471U ,    2147483647U            };
     //--------------------------------------------------------------------------//
     //      Static variable with  1111111111 , 0000000001 , 0000000011 ...      //
     //--------------------------------------------------------------------------//
unsigned int bools::mask0011[word_size] = 
   {   4294967295U ,             1U ,             3U ,             7U ,
               15U ,            31U ,            63U ,           127U ,
              255U ,           511U ,          1023U ,          2047U ,
             4095U ,          8191U ,         16383U ,         32767U ,
            65535U ,        131071U ,        262143U ,        524287U ,
          1048575U ,       2097151U ,       4194303U ,       8388607U ,
         16777215U ,      33554431U ,      67108863U ,     134217727U ,
        268435455U ,     536870911U ,    1073741823U ,    2147483647U         };
     //--------------------------------------------------------------------------//
     //      Static variable with  0000000000 , 1111111110 , 1111111100 ...      //
     //--------------------------------------------------------------------------//
unsigned int bools::mask1100[word_size] = 
   {            0U ,    4294967294U ,    4294967292U ,    4294967288U ,
       4294967280U ,    4294967264U ,    4294967232U ,    4294967168U ,
       4294967040U ,    4294966784U ,    4294966272U ,    4294965248U ,
       4294963200U ,    4294959104U ,    4294950912U ,    4294934528U ,
       4294901760U ,    4294836224U ,    4294705152U ,    4294443008U ,
       4293918720U ,    4292870144U ,    4290772992U ,    4286578688U ,
       4278190080U ,    4261412864U ,    4227858432U ,    4160749568U ,
       4026531840U ,    3758096384U ,    3221225472U ,    2147483648U         };

     //--------------------------------------------------------------------------//
     //                      Output on 'fout' file stream                        //
     //--------------------------------------------------------------------------//
void bools::write(ofstream & fout) const
{
 fout << dimension << "\n";
 for (int i=0; i<words; i++)
   fout << values[i] << "\n";
}
     //--------------------------------------------------------------------------//
     //                      Input from 'fin' file stream                        //
     //--------------------------------------------------------------------------//
void bools::read(ifstream & fin)
{
 int dim;

 fin >> dim;
 create(dim);
 for (int i=0; i<words; i++)
   fin >> values[i];
}
     //--------------------------------------------------------------------------//
     //              Memory space used by 'this' (in memory units)               //
     //--------------------------------------------------------------------------//
int bools::mem_use() const
{
 int answer = 8;
 answer += words * 4;
 return(answer);
}

     //--------------------------------------------------------------------------//
     //                   Debugging option to dump the contents                  //
     //--------------------------------------------------------------------------//
#ifdef _PEPS_DEBUG_
void bools::dump()
{
 ofstream fout;
 char     ans[128];
 int      i;

 cout << "Do you want to see a debug? ";
 cin >> ans;
 if (ans[0] == 'y') {
   cout << "Enter the file name to save the debug: ";
   cin >> ans;
   fout.open(ans);
   if (!fout)
      cerr << "System Error - I Cannot open the file '" << ans << "'\n";
   else {
     fout << "Dumping a bools variable\n\n";
     fout << "States (" << dimension << "):\n";
     for (i=0; i<dimension; i++) {
       fout << i << " : ";
       if ((values[int(i / word_size)] & mask0010[i % word_size]) != all_zero)
         fout << "true\n";
       else
         fout << "false\n";
     }
     fout << "Words (" << words << "):\n";
     for (i=0; i<words; i++)
       fout << i << " : " << values[i] << "\n";
     fout.close();
     if (!fout)
       cerr << "System Error - I Cannot close the file '" << ans << "'\n";
     else
       cout << " :-) debug file '" << ans << "' written\n";
   }
 }
}
#endif
